/*
** Copyright (C) 2003-2006 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogeditnote.h"
#include "utilities.h"
#include "gwrappers.h"
#include "usfmtools.h"
#include "stylesheetutils.h"
#include <gdk/gdkkeysyms.h>
#include "help.h"
#include "shortcuts.h"
#include "biblenotes.h"


EditNoteDialog::EditNoteDialog (Editor * editor)
{
  // Save variables.
  myeditor = editor;

  // Build dialog.
  Shortcuts shortcuts (0);
  
  editnotedialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (editnotedialog), "Edit note");
  gtk_window_set_position (GTK_WINDOW (editnotedialog), GTK_WIN_POS_CENTER_ON_PARENT);

  dialog_vbox1 = GTK_DIALOG (editnotedialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  hbox1 = gtk_hbox_new (FALSE, 10);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 1);

  label1 = gtk_label_new_with_mnemonic ("Numbering");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);

  GSList *radiobutton_numbering_automatic_group = NULL;
  
  radiobutton_numbering_automatic = gtk_radio_button_new_with_mnemonic (NULL, "Automatic");
  gtk_widget_show (radiobutton_numbering_automatic);
  gtk_box_pack_start (GTK_BOX (hbox1), radiobutton_numbering_automatic, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_numbering_automatic), radiobutton_numbering_automatic_group);
  radiobutton_numbering_automatic_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_numbering_automatic));

  shortcuts.button (radiobutton_numbering_automatic);

  radiobutton_numbering_none = gtk_radio_button_new_with_mnemonic (NULL, "None");
  gtk_widget_show (radiobutton_numbering_none);
  gtk_box_pack_start (GTK_BOX (hbox1), radiobutton_numbering_none, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_numbering_none), radiobutton_numbering_automatic_group);
  radiobutton_numbering_automatic_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_numbering_none));

  shortcuts.button (radiobutton_numbering_none);

  radiobutton_numbering_character = gtk_radio_button_new_with_mnemonic (NULL, "Character");
  gtk_widget_show (radiobutton_numbering_character);
  gtk_box_pack_start (GTK_BOX (hbox1), radiobutton_numbering_character, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_numbering_character), radiobutton_numbering_automatic_group);
  radiobutton_numbering_automatic_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_numbering_character));

  shortcuts.button (radiobutton_numbering_character);

  entry_numbering = gtk_entry_new ();
  gtk_widget_show (entry_numbering);
  gtk_box_pack_start (GTK_BOX (hbox1), entry_numbering, FALSE, FALSE, 0);
  gtk_entry_set_max_length (GTK_ENTRY (entry_numbering), 3);
  gtk_entry_set_width_chars (GTK_ENTRY (entry_numbering), 3);

  hseparator5 = gtk_hseparator_new ();
  gtk_widget_show (hseparator5);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator5, TRUE, TRUE, 0);

  hbox7 = gtk_hbox_new (FALSE, 10);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox7, FALSE, FALSE, 0);

  label4 = gtk_label_new_with_mnemonic ("Type");
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox7), label4, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

  GSList *radiobutton_footnote_group = NULL;

  radiobutton_footnote = gtk_radio_button_new_with_mnemonic (NULL, "Footnote");
  gtk_widget_show (radiobutton_footnote);
  gtk_box_pack_start (GTK_BOX (hbox7), radiobutton_footnote, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_footnote), radiobutton_footnote_group);
  radiobutton_footnote_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_footnote));

  shortcuts.button (radiobutton_footnote);

  radiobutton_endnote = gtk_radio_button_new_with_mnemonic (NULL, "Endnote");
  gtk_widget_show (radiobutton_endnote);
  gtk_box_pack_start (GTK_BOX (hbox7), radiobutton_endnote, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_endnote), radiobutton_footnote_group);
  radiobutton_footnote_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_endnote));

  shortcuts.button (radiobutton_endnote);

  radiobutton_xref = gtk_radio_button_new_with_mnemonic (NULL, "Crossreference");
  gtk_widget_show (radiobutton_xref);
  gtk_box_pack_start (GTK_BOX (hbox7), radiobutton_xref, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_xref), radiobutton_footnote_group);
  radiobutton_footnote_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_xref));

  shortcuts.button (radiobutton_xref);

  dialog_action_area1 = GTK_DIALOG (editnotedialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  help_button (dialog_action_area1, &shortcuts, "");
  
  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (editnotedialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (editnotedialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);
  
  shortcuts.stockbutton (cancelbutton1);
  shortcuts.stockbutton (okbutton1);
  shortcuts.process ();

  g_signal_connect ((gpointer) radiobutton_numbering_automatic, "toggled", G_CALLBACK (on_radiobutton_numbering_toggled), gpointer(this));
  g_signal_connect ((gpointer) radiobutton_numbering_none, "toggled", G_CALLBACK (on_radiobutton_numbering_toggled), gpointer(this));
  g_signal_connect ((gpointer) radiobutton_numbering_character, "toggled", G_CALLBACK (on_radiobutton_numbering_toggled), gpointer(this));
  g_signal_connect ((gpointer) okbutton1, "clicked", G_CALLBACK (on_okbutton1_clicked), gpointer(this));

  // Set GUI.
  gint id = myeditor->note_id_at_cursor_get ();
  if (id) {
    ustring rawnote = myeditor->text_note_get (id);
    ustring opener, caller, closer;
    biblenotes_extract_note_opener_caller_closer (rawnote, opener, caller, closer);
    note_set (biblenotes_get_note_type (opener));
    NumberingType numberingtype;
    ustring character = "*";
    biblenotes_get_note_numbering (caller, numberingtype, character);
    numbering_set (numberingtype);
    gtk_entry_set_text (GTK_ENTRY (entry_numbering), character.c_str());
  }
}


EditNoteDialog::~EditNoteDialog ()
{
  gtk_widget_destroy (editnotedialog);
}


int EditNoteDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (editnotedialog));
}


void EditNoteDialog::on_radiobutton_numbering_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  ((EditNoteDialog *) user_data)->on_radiobutton_numbering (togglebutton);
}


void EditNoteDialog::on_radiobutton_numbering (GtkToggleButton *togglebutton)
{
  gtk_label_set_mnemonic_widget (GTK_LABEL (label1), GTK_WIDGET (togglebutton));
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_numbering_character))) {
    gtk_widget_set_sensitive (entry_numbering, true);
    gtk_widget_grab_focus (entry_numbering);
  } else {
    gtk_widget_set_sensitive (entry_numbering, false);
  }
}


int EditNoteDialog::numbering_get ()
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_numbering_none)))
    return ntNone;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_numbering_character)))
    return ntCharacter;
  return ntAutomatic;
}


void EditNoteDialog::numbering_set (int numbering)
{
  NumberingType numberingtype = (NumberingType) numbering;
  switch (numberingtype) {
    case ntAutomatic:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_numbering_automatic), true);
      on_radiobutton_numbering (GTK_TOGGLE_BUTTON (radiobutton_numbering_automatic));
      break;
    case ntNone:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_numbering_none), true);
      on_radiobutton_numbering (GTK_TOGGLE_BUTTON (radiobutton_numbering_none));
      break;
    case ntCharacter:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_numbering_character), true);
      on_radiobutton_numbering (GTK_TOGGLE_BUTTON (radiobutton_numbering_character));
      break;
  }
}


int EditNoteDialog::note_get ()
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_footnote)))
    return indtFootnote;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_endnote)))
    return indtEndnote;
  return indtCrossreference;
}


void EditNoteDialog::note_set (int note)
{
  NoteType notetype = (NoteType) note;
  switch (notetype) {
    case indtFootnote:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_footnote), true);
      break;
    case indtEndnote:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_endnote), true);
      break;
    case indtCrossreference:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_xref), true);
      break;
  }
}


void EditNoteDialog::on_okbutton1_clicked (GtkButton *button, gpointer user_data)
{
  ((EditNoteDialog *) user_data)->on_okbutton ();
}


void EditNoteDialog::on_okbutton ()
{
  // Get the id of the note at the cursor. Bail out if there is none.
  gint id = myeditor->note_id_at_cursor_get ();
  if (!id) return;
  
  // Get the text of the note at the cursor and extract the various parts of it.
  ustring rawnote = myeditor->text_note_get (id);
  ustring notebody = biblenotes_extract_note_body (rawnote);
  ustring opener, caller, closer;
  biblenotes_extract_note_opener_caller_closer (rawnote, opener, caller, closer);
  
  // Get the old and new notetype.
  // Change the opener to the one for the new note type.
  NoteType oldnotetype = biblenotes_get_note_type (opener);
  NoteType newnotetype = (NoteType) note_get ();
  opener = biblenotes_get_note_marker (newnotetype);
  opener = usfm_get_full_opening_marker (opener);

  // Change the caller, if needed.
  NumberingType numberingtype = (NumberingType) numbering_get ();
  gchar * character = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry_numbering));
  caller = biblenotes_get_note_caller (numberingtype, character);  

  /*
  Change the main body of the footnote.
  There are some markers that only belong to foot/endnotes,
  and other ones that only belong to crossreferences.
  Switch between these markers, if that is needed.
  
  Markers for foot- and endnotes:
  fr Reference
  fk Keyword
  fq  Quotation
  fqa Alternative translation.
  fl  Labelled Text
  fp  Additional Paragraph
  fv  Verse in Text
  ft  Text
  fdc Apocrypha

  Markers for crossreferences:
  xo  Origin Reference
  xk  Keyword
  xq  Quotation
  xt  Targets
  xdc Apocrypha    

  */
  enum SwitchNoteType {sntNone, sntToFootEndNote, sntToXref};
  SwitchNoteType switch_note_type = sntNone;
  if ((oldnotetype == indtCrossreference) && (newnotetype != oldnotetype))
    switch_note_type = sntToFootEndNote;
  if ((newnotetype == indtCrossreference) && (newnotetype != oldnotetype))
    switch_note_type = sntToXref;
  switch (switch_note_type) {
    case sntNone: 
    {
      break;
    }
    case sntToFootEndNote:
    {
      /*
      xo  -> fr
      xk  -> fk
      xq  -> fq
      xt  -> ft
      xdc -> fdc      
      */
      biblenotes_switch_markers (notebody, "xo", "fr");
      biblenotes_switch_markers (notebody, "xk", "fk");
      biblenotes_switch_markers (notebody, "xq", "fq");
      biblenotes_switch_markers (notebody, "xt", "ft");
      biblenotes_switch_markers (notebody, "xdc", "fdc");
      break;
    }
    case sntToXref:
    {
      /*
      Switches:
      fr  -> xo
      fk  -> xk
      fq  -> xq
      fqa -> xt
      fl  -> xt
      fp  -> xt
      fv  -> xt
      ft  -> xt
      fdc -> xdc
      */
      biblenotes_switch_markers (notebody, "fr", "xo");
      biblenotes_switch_markers (notebody, "fk", "xk");
      biblenotes_switch_markers (notebody, "fq", "xq");
      biblenotes_switch_markers (notebody, "fqa", "xt");
      biblenotes_switch_markers (notebody, "fl", "xt");
      biblenotes_switch_markers (notebody, "fp", "xt");
      biblenotes_switch_markers (notebody, "fv", "xt");
      biblenotes_switch_markers (notebody, "ft", "xt");
      biblenotes_switch_markers (notebody, "fdc", "xdc");
      break;
    }
  }

  // Change the closer to the one for the new note type.
  closer = biblenotes_get_note_marker (newnotetype);
  closer = usfm_get_full_closing_marker (closer);

  // Remove the current note.
  myeditor->footnote_caller_remove (false);
  
  // Assemble the new note.
  rawnote = opener;
  rawnote.append (caller);
  rawnote.append (" ");
  rawnote.append (notebody);
  rawnote.append (closer);
  
  // Insert the new note in the editor and reformat.
  gtk_text_buffer_insert_at_cursor (myeditor->textbuffer_text, rawnote.c_str(), -1);
  myeditor->format ();
  myeditor->footnote_caller_scroll_to ();
}
