/*
** Copyright (C) 2003-2006 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#ifndef INCLUDED_EDITOR_H
#define INCLUDED_EDITOR_H


#include "libraries.h"
#include <glib.h>
#include <gtk/gtk.h>
#include "editorobject.h"
#include "reference.h"


class Editor
{
public:
  Editor (GtkWidget * notebook, const ustring& project_in);
  ~Editor ();
  void book_set (unsigned int book_in);
  void chapter_load (unsigned int number, vector<ustring> * lines, bool clear_undo_stack = true);
  void chapter_save ();
  ustring verse_number_get ();
  void footnote_body_scroll_to ();
  void footnote_caller_scroll_to ();
  bool footnote_caller_remove (bool backspace);
  void footnote_remove_empty_ones ();
  ustring text_get_selection ();
  ustring text_get_all ();
  void format ();
  void show_quick_references (GtkWidget * textview, const ustring& project, unsigned int book, bool reread);
  GtkWidget * main_box;
  GtkWidget * button_close;
  GtkWidget * vpaned;
  GtkWidget * vbox_editor;
  // GtkWidget * label_project;
  GtkWidget * scrolledwindow_text;
  GtkWidget * textview_text;
  GtkTextBuffer * textbuffer_text;
  GtkTextTag * text_line_tag;
  GtkWidget * new_verse_signal;
  GtkWidget * scrolledwindow_notes;
  GtkWidget * textview_notes;
  GtkTextBuffer * textbuffer_notes;
  GtkTextTag * notes_line_tag;
  vector<EditorObject> editorobjects;
  bool focus_programmatically_being_grabbed;
  Reference current_reference;
  bool go_to_new_reference_highlight;
  void on_textview_footnotes_erase ();
  ustring word_double_clicked_text;
  GtkWidget * word_double_clicked_signal;
  
  // Undo/redo
  void un_re_do_clear ();
  void un_re_do_text_changed ();
  void undo ();
  void redo ();
  bool can_undo ();
  bool can_redo ();
  
  void set_font ();

  void position_cursor_at_verse (const ustring& cursorposition, bool focus);
  ustring position_cursor_at_verse_cursorposition;
  bool position_cursor_at_verse_focus;
  static bool position_cursor_at_verse_postponer_handler (gpointer user_data);
  void position_cursor_at_verse_executer ();
  GtkTextTag * reference_tag;

  void quick_references_textview_set (GtkWidget * widget);
  GtkWidget * quick_references_textview;

  ustring project;
  unsigned int book;
  unsigned int chapter;

  ustring text_note_get (gint id);
  gint note_id_at_cursor_get ();
private:
  void load_text (vector<ustring> & lines);
  void format_notes ();
  ustring text_get (GtkTextIter * startiter, GtkTextIter * enditer);
  map <int, ustring> text_note_get_all ();
  ustring end_note_signature (EditorObjectType objecttype);
  static void on_textview_text_move_cursor (GtkTextView * textview, GtkMovementStep step, gint count, gboolean extend_selection, gpointer user_data);
  void on_textview_text_cursor_moved_delayer ();
  static bool on_textview_text_cursor_moved_delayer_handler (gpointer user_data);
  void on_textview_text_cursor_moved ();
  static void on_textview_text_grab_focus (GtkWidget * widget, gpointer user_data);
  void on_grab_focus ();
  static bool on_grab_focus_delayer_timeout (gpointer data);
  bool on_grab_focus_delayed_handler ();
  int previous_focused_note_id;

  // Undo/redo
  static bool un_re_do_on_timeout (gpointer data);
  void un_re_do_timeout ();
  bool un_re_do_tick ();
  void un_re_do_store_snapshot ();
  void un_re_do_clear_internal ();
  void un_re_do_store_cursorpositions ();
  void un_re_do_cursorpositions_set (int textposition, int notesposition);
  int un_re_do_previous_cursorposition_text;
  vector <int> un_re_do_cursorpositions_text;
  int un_re_do_previous_cursorposition_notes;
  vector <int> un_re_do_cursorpositions_notes;
  ustring un_re_do_previous_snapshot;
  vector <ustring> un_re_do_snapshots;
  unsigned int un_re_do_pointer;
  unsigned int un_re_do_timer;
  static void on_textbuffer_edit_insert_text (GtkTextBuffer * textbuffer, GtkTextIter * arg1, gchar * arg2, gint arg3, gpointer user_data);
  static void on_textbuffer_edit_delete_text (GtkTextBuffer * textbuffer, GtkTextIter * arg1, GtkTextIter * arg2, gpointer user_data);
  void on_textbuffer_edit_insert_delete ();  

  // Automatic save.
  static bool mainwindow_on_save_timeout (gpointer data);
  bool on_save_timeout ();


  static gboolean on_text_motion_notify_event (GtkWidget *textview, GdkEventMotion *event, gpointer user_data);
  gboolean text_motion_notify_event (GtkWidget *textview, GdkEventMotion *event);
  static void on_text_event_after (GtkWidget *widget, GdkEvent *event, gpointer user_data);
  void text_event_after (GtkWidget *textview, GdkEvent *ev);
  void text_edit_if_link (GtkWidget *textview, GtkTextIter *iter);
  static gboolean text_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data);
  gboolean on_text_key_press_event (GtkWidget *widget, GdkEventKey *event);
  static void on_textview_footnotes_move_cursor (GtkTextView * textview, GtkMovementStep step, gint count, gboolean extend_selection, gpointer user_data);
  void on_textview_footnotes_cursor_moved ();
  static void on_textview_footnotes_grab_focus (GtkWidget * widget, gpointer user_data);
  void on_textview_footnotes_focus_grabbed ();
  static bool on_textview_footnotes_focus_grabbed_delayed (gpointer data);
  void on_textview_footnotes_focus_grabbed_timeout ();
  static void on_textview_footnotes_backspace (GtkTextView *textview, gpointer user_data);
  static void on_textview_footnotes_cut_clipboard (GtkTextView *textview, gpointer user_data);
  static void on_textview_footnotes_delete_from_cursor (GtkTextView *textview, GtkDeleteType type, gint count, gpointer user_data);
  static void on_textbuffer_footnotes_changed (GtkEditable * editable, gpointer user_data);
  void on_textbuffer_footnotes ();
  static gboolean on_textview_edit_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
  static gboolean on_textview_footnotes_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
  void on_texteditor_click (GtkWidget * widget, GdkEventButton *event);

  // Some event ids.
  guint textview_text_cursor_moved_delayer_event_id;
  guint grab_focus_event_id;
  guint position_cursor_at_verse_event_id;
  guint undo_redo_event_id;
  guint save_timeout_event_id;
  
  static bool on_widget_creation_timeout (gpointer data);
  void on_widget_creation ();
  
  static void on_vpaned_editor_size_allocate (GtkWidget *widget, GdkRectangle *allocation, gpointer user_data);
  int vpaned_previous_height;
  void on_vpaned_editor_size_allocated (GdkRectangle *allocation);
  void bible_notes_window_set_size ();
  bool bible_notes_window_setting_size;
  void bible_notes_window_store_size ();
  unsigned int vpaned_previous_handle_position;
};


#endif
