/*
** Copyright (C) 2003-2006 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include <glib.h>
#include "formatter.h"
#include "xep.h"
#include "gwrappers.h"
#include "directories.h"
#include "shell.h"
#include "session.h"


bool formatter_present (GtkWidget * window)
// Returns whether a script to run a formatter can be found.
// Gives message if not found.
{
  return xep_present (window);
}


bool formatter_convert_to_pdf (const ustring& xslfo, const ustring& pdf, NoteCaller * footnotecaller, NoteCaller * crossreferencecaller)
// Converts file "xslfo" to file "pdf", and returns the result.
{
  // Remove any previously created pdf file, lest it would be shown in case of 
  // conversion errors.
  unlink (pdf.c_str());

  // Formatting is done in three stages:
  // 1. Convert from the xslfo file to the xep intermediate format.
  // 2. If need be, modify that intermediate file.
  // 3. Convert from the xep intermediate file to pdf output.  
  ustring xep_filename = xslfo + ".xep";

  // Cancel variable.
  bool cancel = false;
  
  // Start the first formatting process in the background.
  if (!xep_convert_from_fo_to_xep (xslfo, xep_filename, cancel))
    return false;
  if (cancel) 
    return true;
    
  // Do an interim stage: restarting footnote numbering each page.
  xep_convert_footnote_callers (xep_filename, footnotecaller, crossreferencecaller);
  
  // Do another interim stage: calculating the length of elastics.
  XepElastics xep_elastics (xep_filename);
  
  // Do the last stage of the conversion.
  if (!xep_convert_from_xep_to_pdf (xep_filename, pdf, cancel))
    return false;
  if (cancel)
    return true;

  // Check the pdf file is there.
  if (!g_file_test (pdf.c_str(), G_FILE_TEST_IS_REGULAR))
    return false;
  
  // Everything's ok.
  return true;
}
