/*
** Copyright (C) 2003-2006 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#ifndef INCLUDED_PROJECTCONFIGURATION_H
#define INCLUDED_PROJECTCONFIGURATION_H


#include "libraries.h"
#include "constants.h"
#include "sqlite3.h"
#include "generalconfig.h"


void project_configuration_vacuum (const ustring& project, unsigned int starttime);
ustring project_configuration_filename (const ustring& project);
void upgrade_project_configuration (const ustring& project);


class ProjectConfiguration
// Manages program settings.
{
public:
  ProjectConfiguration (ustring project);
  ~ProjectConfiguration ();
  ustring value_get (char * key, const ustring& standard);
  void value_set (char * key, const ustring& value);
  vector<ustring> valuelist_get (char * key);
  void valuelist_set (char * key, const vector<ustring>& value);
private:
  char * stylesheet_key ();
public:
  ustring stylesheet ();
  void stylesheet_set (ustring value);
private:
  char * versification_key ();
public:
  ustring versification ();
  void versification_set (ustring value);
private:
  char * printing_fonts_key ();
public:
  vector<ustring> printing_fonts ();
  void printing_fonts_set (vector<ustring> value);
private:
  char * printing_font_size_key ();
public:
  int printing_font_size ();
  void printing_font_size_set (int value);
private:
  char * printing_line_height_key ();
public:
  int printing_line_height ();
  void printing_line_height_set (int value);
private:
  char * sword_name_key ();
public:
  ustring sword_name ();
  void sword_name_set (ustring value);
private:
  char * sword_description_key ();
public:
  ustring sword_description ();
  void sword_description_set (ustring value);
private:
  char * sword_about_key ();
public:
  ustring sword_about ();
  void sword_about_set (ustring value);
private:
  char * sword_lcsh_key ();
public:
  ustring sword_lcsh ();
  void sword_lcsh_set (ustring value);
private:
  char * sword_license_key ();
public:
  ustring sword_license ();
  void sword_license_set (ustring value);
private:
  char * sword_version_key ();
public:
  ustring sword_version ();
  void sword_version_set (ustring value);
private:
  char * sword_language_key ();
public:
  ustring sword_language ();
  void sword_language_set (ustring value);
private:
  char * reordered_books_key ();
public:
  vector<ustring> reordered_books ();
  void reordered_books_set (vector<ustring> value);
private:
  char * reordered_includes_key ();
public:
  vector<bool> reordered_includes ();
  void reordered_includes_set (vector<bool> value);
private:
  char * reordered_portions_key ();
public:
  vector<ustring> reordered_portions ();
  void reordered_portions_set (vector<ustring> value);
private:
  char * language_key ();
public:
  ustring language ();
  void language_set (ustring value);
private:
  char * book_order_key ();
public:
  vector<unsigned int> book_order ();
  void book_order_set (vector<unsigned int> value);
private:
  char * editable_key ();
public:
  bool editable ();
  void editable_set (bool value);
private:
  char * backup_incremental_last_time_key ();
public:
  int backup_incremental_last_time ();
  void backup_incremental_last_time_set (int value);
private:
  char * backup_full_key ();
public:
  bool backup_full ();
  void backup_full_set (bool value);
private:
  char * repository_remote_key ();
public:
  bool repository_remote ();
  void repository_remote_set (bool value);
private:
  char * repository_username_key ();
public:
  ustring repository_username ();
  void repository_username_set (ustring value);
private:
  char * repository_password_key ();
public:
  ustring repository_password ();
  void repository_password_set (ustring value);
private:
  char * repository_conflict_key ();
public:
  int repository_conflict ();
  void repository_conflict_set (int value);
private:
  char * changes_last_review_time_key ();
public:
  int changes_last_review ();
  void changes_last_review_set (int value);
private:
  char * changes_since_time_key ();
public:
  int changes_since ();
  void changes_since_set (int value);
private:
  char * vpane_biblenotes_percentage_key ();
public:
  int vpane_biblenotes_percentage ();
  void vpane_biblenotes_percentage_set (int value);
private:
  char * editor_font_default_key ();
public:
  bool editor_font_default ();
  void editor_font_default_set (bool value);
private:
  char * editor_font_name_key ();
public:
  ustring editor_font_name ();
  void editor_font_name_set (ustring value);
private:
  char * editor_default_color_key ();
public:
  bool editor_default_color ();
  void editor_default_color_set (bool value);
private:
  char * editor_normal_text_color_key ();
public:
  int editor_normal_text_color ();
  void editor_normal_text_color_set (int value);
private:
  char * editor_background_color_key ();
public:
  int editor_background_color ();
  void editor_background_color_set (int value);
private:
  char * editor_selected_text_color_key ();
public:
  int editor_selected_text_color ();
  void editor_selected_text_color_set (int value);
private:
  char * editor_selection_color_key ();
public:
  int editor_selection_color ();
  void editor_selection_color_set (int value);
private:
  sqlite3 *db;
  int rc;
  char *error;
  void create (const ustring& filename);
  bool bool_get (char * id, bool standard);
  void bool_set (char * id, bool value);
  int int_get (char * id, int standard);
  void int_set (char * id, int value);
  ustring string_get (char * id, const ustring& standard);
  void string_set (char * id, const ustring& value);
  double double_get (char * id, double standard);
  void double_set (char * id, double value);
  vector<bool> vector_bool_get (char * id);
  void vector_bool_set (char * id, const vector<bool>& value);
  vector<ustring> vector_string_get (char * id);
  void vector_string_set (char * id, const vector<ustring>& value);
  vector<unsigned int> vector_unsigned_int_get (char * id);
  void vector_unsigned_int_set (char * id, const vector<unsigned int>& value);
  GeneralConfiguration genconfig;
};


#endif
